/*
ARM7 code for cf bench
based on ARM7 code for RAC

Copyright 2007 Damian Yerrick <pinoandchester@pineight.com>

This work is provided 'as-is', without any express or implied
warranty.  In no event will the authors be held liable for any
damages arising from the use of this work.

Permission is granted to anyone to use this work for any purpose,
including commercial applications, and to alter it and redistribute
it freely, subject to the following restrictions:

1. The origin of this work must not be misrepresented; you must
   not claim that you wrote the original work. If you use
   this work in a product, an acknowledgment in the product
   documentation would be appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original work.
3. This notice may not be removed or altered from any source
   distribution.

"Source" is the preferred form of a work for making changes to it.

*/
/*
Based on libnds ARM7 template by wintermute
*/

#include <nds.h>
#include <stdlib.h>
#include "talkback.h"

//////////////////////////////////////////////////////////////////////


#define TOUCH_CAL_X1 (*(vs16*)0x027FFCD8)
#define TOUCH_CAL_Y1 (*(vs16*)0x027FFCDA)
#define TOUCH_CAL_X2 (*(vs16*)0x027FFCDE)
#define TOUCH_CAL_Y2 (*(vs16*)0x027FFCE0)
#define SCREEN_WIDTH    256
#define SCREEN_HEIGHT   192
s32 TOUCH_WIDTH;
s32 TOUCH_HEIGHT;
s32 TOUCH_OFFSET_X;
s32 TOUCH_OFFSET_Y;
u32 heartbeat = 0;

//////////////////////////////////////////////////////////////////////



void runSound(void) {
}


/**
 * Sets the power management chip on the ARM7 SPI bus.
 */
__attribute__((noinline)) void PM_SetRegister(int reg, int control)
{

  SerialWaitBusy();

  REG_SPICNT = SPI_ENABLE | SPI_DEVICE_POWER | SPI_BAUD_1MHz | SPI_CONTINUOUS;
  REG_SPIDATA = reg;

  SerialWaitBusy();

  REG_SPICNT = SPI_ENABLE | SPI_DEVICE_POWER | SPI_BAUD_1MHz;
  REG_SPIDATA = control;
}

void powerOffDS(void)
{
  PM_SetRegister(0, 1 << 6);//6 DS power (0: on, 1: shut down!)
}



int vcount;
touchPosition first,tempPos;

//---------------------------------------------------------------------------------
void VcountHandler() {
//---------------------------------------------------------------------------------
	static int lastbut = -1;
	
	uint16 but=0, x=0, y=0, xpx=0, ypx=0, z1=0, z2=0;

	but = REG_KEYXY;

	if (!( (but ^ lastbut) & (1<<6))) {
 
		tempPos = touchReadXY();

		if ( tempPos.x == 0 || tempPos.y == 0 ) {
			but |= (1 <<6);
			lastbut = but;
		} else {
			x = tempPos.x;
			y = tempPos.y;
			xpx = tempPos.px;
			ypx = tempPos.py;
			z1 = tempPos.z1;
			z2 = tempPos.z2;
		}
		
	} else {
		lastbut = but;
		but |= (1 <<6);
	}

	if ( vcount == 80 ) {
		first = tempPos;
	} else {
		if (	abs( xpx - first.px) > 10 || abs( ypx - first.py) > 10 ||
				(but & ( 1<<6)) ) {

			but |= (1 <<6);
			lastbut = but;

		} else { 	
			IPC->mailBusy = 1;
			IPC->touchX			= x;
			IPC->touchY			= y;
			IPC->touchXpx		= xpx;
			IPC->touchYpx		= ypx;
			IPC->touchZ1		= z1;
			IPC->touchZ2		= z2;
			IPC->mailBusy = 0;
		}
	}
	IPC->buttons		= but;
	vcount ^= (80 ^ 130);
	SetYtrigger(vcount);

}

//---------------------------------------------------------------------------------
void VblankHandler(void) {
//---------------------------------------------------------------------------------

    volatile P8A7Talkback *tb = (P8A7Talkback *)IPC->soundData;
    int axeFreq = -1;
   
    if (tb) {
      if (tb->needPowerOff) {
        tb->needPowerOff = 0;
        powerOFF(POWER_SOUND);
        powerOffDS();
      }
      if (tb->needNote) {
        axeFreq = tb->needNote * 8;
      }
    }

    //axe(axeFreq, 48);
    //runSound();

    ++heartbeat;
 
}

//---------------------------------------------------------------------------------
int main(int argc, char ** argv) {
//---------------------------------------------------------------------------------

	IPC->soundData = 0;
	rtcReset();
    rtcGetTimeAndDate((uint8 *)&(IPC->time.rtc.year));

	//enable sound
	powerON(POWER_SOUND);
	SOUND_CR = SOUND_ENABLE | SOUND_VOL(0x7F);

	irqInit();
	irqSet(IRQ_VBLANK, VblankHandler);
	SetYtrigger(80);
	vcount = 80;
	irqSet(IRQ_VCOUNT, VcountHandler);
	irqEnable(IRQ_VBLANK | IRQ_VCOUNT);

	// Keep the ARM7 idle
	while (1) swiWaitForVBlank();
}

//////////////////////////////////////////////////////////////////////
